# Tutorial 20: Semantic Pointer Actions

# A complex cognitive system needs a method to select and perform actions.
# In particular, it is useful to have a model that can do one thing at
# a time, sequentially.  There is significant psychological data indicating
# that people have such a system, providing a serial bottleneck of
# cognition.
#
# In mammals, this system is often identified with the Basal Ganglia.
# We have shown that a Nengo model of the Basal Ganglia acting as
# an action selection network matches well to human data:
# http://compneuro.uwaterloo.ca/files/publications/stewart.2012a.pdf
# "Stewart TC, Bekolay T and Eliasmith C. Learning to select actions with
#  spiking neurons in the basal ganglia. Front. Neurosci. (6)2, 2012."
#
# This model has been built into the Nengo SPA system for use in your
# models.  To use it, for every action you need to define some function
# that computes the utility of the action (how good the action is right
# now) and what the effect of the action should be (what to do when
# the action is chosen).  The Basal Ganglia model will monitor the
# utilities of each action and select the one with the highest value.
#
# The following code defines four actions.  The utilities look like
# this: "dot(vision, DOG)".  This means that the utility of the action
# is the dot product of whatever vector is in the vision State and
# the ideal vector for DOG.  This will be a value near 1 if you see a
# dog, and will be near zero otherwise.  The closer what you see is to
# a dog, the higher the utility.  The effect of the action is simply
# to set the speech State to the vector for BARK ("speech=BARK").
#
# Try running the model.  Use "Set value" on the vision graph to set the
# input to DOG.  The speech output should become BARK.  Try other inputs,
# such as CAT or COW.  You can also try a combination, such as
# RAT+0.5*COW.  The system will (almost always) select a single action
# to perform, and that will be the action with the highest utility at that
# moment.  If there is no input, or the input is not sufficiently similar 
# to any of the four animals it knows, then the speech State is set to 
# zero ("speech=0").
#
# The input graph above the Basal Ganglia shows the utilities of the
# four actions.  The output graph above the thalamus shows which
# of the four actions are selected.  The thalamus controls the routing of
# information between brain areas, so here it is used to implement the
# effects of the actions defined in the Basal Ganglia.

import nengo
import nengo.spa as spa

D = 32  # the dimensionality of the vectors

model = spa.SPA()
with model:
    model.vision = spa.State(D)

    model.speech = spa.State(D)

    actions = spa.Actions(
        'dot(vision, DOG) --> speech=BARK',
        'dot(vision, CAT) --> speech=MEOW',
        'dot(vision, RAT) --> speech=SQUEAK',
        'dot(vision, COW) --> speech=MOO',
        '0.5 --> speech=0'
        )

    model.bg = spa.BasalGanglia(actions)
    model.thalamus = spa.Thalamus(model.bg)



# #Nengo Example: Addition

# In this example, we will construct a network that adds two inputs. The
# network utilizes two communication channels into the same neural population.
# Addition is thus somewhat free, since the incoming currents from
# different synaptic connections interact linearly (though two inputs dont
# have to combine in this way: see the combining demo).

import nengo
model = nengo.Network()
with model:
    # Create 3 ensembles each containing 100 leaky integrate-and-fire neurons
    a = nengo.Ensemble(n_neurons=100, dimensions=1)
    b = nengo.Ensemble(n_neurons=100, dimensions=1)
    c = nengo.Ensemble(n_neurons=100, dimensions=1)

    # Create input nodes representing constant values
    stim_a = nengo.Node(0.5)
    stim_b = nengo.Node(0.3)

    # Connect the input nodes to the appropriate ensembles
    nengo.Connection(stim_a, a)
    nengo.Connection(stim_b, b)

    # Connect input ensembles a and b to output ensemble c
    nengo.Connection(a, c)
    nengo.Connection(b, c)



# # Nengo Example: Integrator
#
# This demo implements a one-dimensional neural integrator.
#
# This is the first example of a recurrent network in the demos. It shows how
# neurons can be used to implement stable dynamics. Such dynamics are important
# for memory, noise cleanup, statistical inference, and many other dynamic
# transformations.
#
# Note that since the integrator constantly sums its input, it will saturate
# quickly if you leave the input non-zero. This makes it  clear that neurons
# have a finite range of representation. Such saturation effects can be
# exploited to perform useful computations (e.g. soft normalization).

import nengo

tau = 0.1

model = nengo.Network()

with model:
    integrator = nengo.networks.Integrator(
        tau, n_neurons=100, dimensions=1)
    stim = nengo.Node(0)

    nengo.Connection(stim, integrator.input, synapse=tau)



# Tutorial 25: Parsing Simple Commands

# In this tutorial, we use both the ability to combine structured information
# and the ability to make complex actions to implement a simple two-word
# parsing system.  This model has a single visual input (vision) and you can
# present to it two-word commands.  For example, you can say WRITE followed
# by HI.  It will remember the two terms, store them, and then respond
# appropriately by sending HI to its hand.  It can also SAY BYE, where the
# vector for BYE will be set to the speech system.

# The parsing process is implemented with the first two actions.  The first
# action looks for verbs (WRITE or SAY) and sends them into the memory for
# verbs. The second action looks for nouns (YES or NO or HI or BYE or OK) and
# sends them to the memory for nouns.  The noun stored in the noun memory binds
# with the semantic pointer NOUN and the verb stored in the verb memory binds
# with the semantic pointer VERB in the phrase state.  This binding is
# implemented with cortical actions. Notice that the actions can look for
# multiple different things at the same time by using the + operation.
# Overall, this shows how we can use a single input modality (vision) and treat
# the information in different ways as appropriate.

# The last two actions deal with recognizing and executing actions.  The
# first one looks for the VERB WRITE, and if it sees this it will take
# whatever is in phrase, extract out the NOUN, and send that to the hand.
# Furthermore, it will only do this if it doesn't currently see anything in
# vision (via the subtraction).  This is to make sure it waits until it is
# getting no more visual input before responding.

# To test the model, try presenting WRITE to the vision State.  The phrase
# should fill with WRITE*VERB.  Now change the vision input to HI.  The phrase
# should now fill with NOUN*HI.  If you get rid of the visual input (by setting
# the value to nothing), it shuold sent HI to the hand.  If you try the same
# thing with SAY it should send the result to speech.

import nengo
import nengo.spa as spa

D = 32  # the dimensionality of the vectors

model = spa.SPA()
with model:
    # perceptual input
    model.vision = spa.State(D)

    # linguistic components
    model.phrase = spa.State(D)
    model.verb = spa.State(D, feedback=1)
    model.noun = spa.State(D, feedback=1)

    # motor components
    model.speech = spa.State(D)
    model.hand = spa.State(D)

    actions = spa.Actions(
        'dot(vision, WRITE+SAY) --> verb=vision',
        'dot(vision, YES+NO+HI+BYE+OK) --> noun=vision',
        'dot(phrase, VERB*WRITE) - 2*dot(vision, WRITE+SAY+YES+NO+HI+BYE+OK)'
            '--> hand=phrase*~NOUN',
        'dot(phrase, VERB*SAY) - 2*dot(vision, WRITE+SAY+YES+NO+HI+BYE+OK)'
            '--> speech=phrase*~NOUN',
        )

    model.bg = spa.BasalGanglia(actions)
    model.thalamus = spa.Thalamus(model.bg)

    cortical_actions = spa.Actions(
        'phrase = verb*VERB + noun*NOUN'
    )

    model.cortical = spa.Cortical(cortical_actions)



import copy

import nengo
import nengo.spa as spa
import numpy as np

from nengo_gui.components.component import Component
from nengo_gui.components.spa_plot import SpaPlot


class Pointer(SpaPlot):
    config_defaults = dict(show_pairs=False, **Component.config_defaults)
    def __init__(self, obj, **kwargs):
        super(Pointer, self).__init__(obj, **kwargs)

        # the semantic pointer value as set by the user in the GUI
        # a value of 'None' means do not override
        self.override_target = None

        # The white list indicates the networks whose user-defined
        # over-ride value can be inserted on the input
        # thus the value loops in from the output to the input.
        # All other networks have their value inserted on the output.
        # Looping-in has the advantage of actually changing the
        # neural activity of the population, rather than just changing
        # the output.
        self.loop_in_whitelist = [spa.Buffer, spa.Memory, spa.State]

        self.node = None
        self.conn1 = None
        self.conn2 = None

    def add_nengo_objects(self, page):
        with page.model:
            output = self.obj.outputs[self.target][0]
            self.node = nengo.Node(self.gather_data,
                                   size_in=self.vocab_out.dimensions,
                                   size_out=self.vocab_out.dimensions)
            self.conn1 = nengo.Connection(output, self.node, synapse=0.01)
            loop_in = type(self.obj) in self.loop_in_whitelist
            if loop_in and self.target == 'default':
                input = self.obj.inputs[self.target][0]
                self.conn2 = nengo.Connection(self.node, input, synapse=0.01)
            else:
                self.conn2 = nengo.Connection(self.node, output, synapse=0.01)

    def remove_nengo_objects(self, page):
        page.model.connections.remove(self.conn1)
        page.model.connections.remove(self.conn2)
        page.model.nodes.remove(self.node)

    def gather_data(self, t, x):
        vocab = self.vocab_out
        key_similarities = np.dot(vocab.vectors, x)
        over_threshold = key_similarities > 0.01
        matches = zip(key_similarities[over_threshold], 
                      np.array(vocab.keys)[over_threshold])
        if self.config.show_pairs:
            self.vocab_out.include_pairs = True
            pair_similarities = np.dot(vocab.vector_pairs, x)
            over_threshold = pair_similarities > 0.01
            pair_matches = zip(pair_similarities[over_threshold],
                               np.array(vocab.key_pairs)[over_threshold])
            matches += pair_matches

        text = ';'.join(['%0.2f%s' % (sim, key) for sim, key in matches])

        # msg sent as a string due to variable size of pointer names
        msg = '%g %s' % (t, text)
        self.data.append(msg)
        if self.override_target is None:
            return np.zeros(self.vocab_out.dimensions)
        else:
            v = (self.override_target.v - x) * 3
            return v

    def update_client(self, client):
        while len(self.data) > 0:
            data = self.data.popleft()
            client.write(data, binary=False)

    def javascript(self):
        info = dict(uid=id(self), label=self.label)
        json = self.javascript_config(info)
        return 'new Nengo.Pointer(main, sim, %s);' % json

    def code_python_args(self, uids):
        return [uids[self.obj], 'target=%r' % self.target]

    def message(self, msg):
        if msg == ':empty:':
            self.override_target = None
        elif msg[0:12] == ':check only:':
            if len(msg) == 12:
                self.data.append("good_pointer")
            else:                
                vocab = copy.deepcopy(self.vocab_out)
                try:
                    vocab.parse(msg[12:])
                    self.data.append("good_pointer")
                except:
                    self.data.append("bad_pointer")
        else:
            # The message value is the new value for the output of the pointer
            try:
                self.override_target = self.vocab_out.parse(msg)
            except:
                self.override_target = None



import contextlib
import importlib
import os
import threading
import traceback
import sys
from nengo.utils.compat import StringIO


# list of Simulators to check for
known_modules = ['nengo', 'nengo_ocl', 'nengo_distilled',
                 'nengo_brainstorm', 'nengo_spinnaker']


def discover_backends():
    found_modules = {}
    for name in known_modules:
        try:
            mod = importlib.import_module(name)
        except Exception as e:
            # TODO only ignore ImportErrors "No module named ...", display
            # other errors to the user as they might help debugging broken
            # backend installations
            continue
        found_modules[name] = mod
    return found_modules


class StartedSimulatorException(Exception):
    pass


class StartedGUIException(Exception):
    pass


# create a wrapper class that will throw an exception if we are
# currently executing a script
def make_dummy(cls):
    class DummySimulator(cls):
        def __init__(self, *args, **kwargs):
            if is_executing():
                raise StartedSimulatorException()
            super(DummySimulator, self).__init__(*args, **kwargs)
    return DummySimulator


# thread local storage for storing whether we are executing a script
flag = threading.local()


def is_executing():
    return getattr(flag, 'executing', False)


def determine_line_number(filename='<string>'):
    '''Checks stack trace to determine the line number we are currently at.

    The default filname argument of "<string>" indicates it should only
    pay attention to the line numbers in the main evaluated script (which
    is evaluated using exec(), so it doesn't have a normal filename).
    '''

    exc_type, exc_value, exc_traceback = sys.exc_info()
    if exc_traceback is not None:
        ex_tb = traceback.extract_tb(exc_traceback)
        for fn, line, function, code in reversed(ex_tb):
            if fn == filename:
                return line

    # if we can't find it that way, parse the text of the stack trace
    #  note that this is required for indentation errors and other syntax
    #  problems
    trace = traceback.format_exc()
    pattern = 'File "%s", line ' % filename
    index = trace.find(pattern)
    if index >=0:
        text = trace[index + len(pattern):].split('\n', 1)[0]
        if ',' in text:
            text = text.split(',', 1)[0]
        line = int(text)
        return line
    return None


class ExecutionEnvironment(object):
    def __init__(self, filename, allow_sim=False):
        if filename is None:
            self.directory = None
        else:
            self.directory = os.path.dirname(filename)
        self.added_directory = None
        self.allow_sim = allow_sim
    def __enter__(self):
        if self.directory is not None and self.directory not in sys.path:
            sys.path.insert(0, self.directory)
            self.added_directory = self.directory

        self.stdout = StringIO()

        flag.executing = True
        self.simulators = {}

        # add hooks to record stdout

        sys.stdout = self.stdout

        if not self.allow_sim:
            for mod in discover_backends().values():
                self.simulators[mod] = mod.Simulator
                mod.Simulator = make_dummy(mod.Simulator)

    def __exit__(self, exc_type, exc_value, traceback):
        for mod, cls in self.simulators.items():
            mod.Simulator = cls
        flag.executing = False

        sys.stdout = sys.__stdout__

        if not self.allow_sim:
            if self.added_directory is not None:
                sys.path.remove(self.added_directory)
                self.added_directory = None



#!/usr/bin/env python
#
# This code is part of Grandalf
#  Copyright (C) 2014 Axel Tillequin (bdcht3@gmail.com) and others
# published under GPLv2 license or EPLv1 license
# Contributor(s): Axel Tillequin, Fabio Zadrozny

'''
Some utilities to interact with networkx.
'''

from grandalf.graphs import Graph, Vertex, Edge


def convert_grandalf_graph_to_networkx_graph(G):
    '''
    Converts a grandalf graph to a networkx graph.
    
    Note that the edge concept is the same, but a vertex in grandalf is called a node in networkx.
    '''
    import networkx
    networkx_graph = networkx.MultiDiGraph()
    for v in G.V():
        networkx_graph.add_node(v.data)
        
    for e in G.E():
        networkx_graph.add_edge(e.v[0].data, e.v[1].data)
                            
    return networkx_graph
    
def convert_nextworkx_graph_to_grandalf(G):
    '''
    Converts a networkx graph to a grandalf graph.
    
    Note that the edge concept is the same, but a vertex in grandalf is called a node in networkx.
    '''
    V = []
    data_to_V = {}
    for x in G.nodes():
        vertex = Vertex(x)
        V.append(vertex)
        data_to_V[x] = vertex
    
    E = [Edge(data_to_V[xy[0]], data_to_V[xy[1]], data=xy) for xy in G.edges()]
    g = Graph(V, E)
    return g



"""Nengo GUI version information.

We use semantic versioning (see http://semver.org/).
Additionally, '-dev' will be added to the version unless the code base
represents a release version. Commits for which the version doesn't have
'-dev' should be git tagged with the version.
"""

name = "nengo_gui"
version_info = (0, 1, 5)  # (major, minor, patch)
dev = False

version = "{v}{dev}".format(v='.'.join(str(v) for v in version_info),
                            dev='-dev' if dev else '')



import pytest
import nengo
from nengo import spa

from nengo_gui.components.spa_plot import SpaPlot

def test_applicable_targets():
    # Only return non-scalar targets
    model = spa.SPA()

    with model:
        model.comp = spa.Compare(4)
        assert SpaPlot.applicable_targets(model.comp) == []

        model.state = spa.State(4)
        res = SpaPlot.applicable_targets(model.state)
        assert res == list(model.state.outputs.keys())



import importlib
import json
import logging
import os
import socket
import threading
import time
import traceback

import nengo

import nengo_gui
import nengo_gui.user_action
import nengo_gui.config


class Page(object):
    """A handler for a singe page of the nengo_gui.

    Parameters
    ----------

    gui : nengo_gui.GUI
        The main GUI
    filename : str
        The filename to open.  If this is the same as gui.filename
        then it will use the existing gui.model and gui.locals
        (if available).  Otherwise, the file will be executed to generate
        the model
    reset_cfg : bool, optional
        If True, the existing .cfg file will be erased
    """

    # Some Simulators can only have one instance running at a time
    singleton_sims = dict(nengo_spinnaker=None)

    def __init__(self, gui, filename, reset_cfg=False):
        self.gui = gui
        self.backend = gui.backend

        self.code = None     # the code for the model
        self.model = None    # the nengo.Network
        self.locals = None   # the locals() dictionary after executing
        self.last_good_locals = None # the locals dict for the last time
                                     # this script was run without errors
        self.error = None    # any error message generated
        self.stdout = ''     # text sent to stdout during execution

        self.changed = False   # has the model been changed?
        self.paused = False    # is the simulation paused
        self.finished = False  # should this Page be shut down
        self._sim = None       # the current nengo.Simulator
        self.rebuild = False   # should the model be rebuilt

        self.code = None       # the source code currently displayed
        self.error = None      # any execute or build error
        self.stdout = ''       # text printed during execute+build

        self.undo_stack = []
        self.redo_stack = []

        # placeholders for attributes that will be created by self.load()
        self.name_finder = None    # NameFinder from nengo objects to text
        self.default_labels = None # dict of names to use for unlabelled objs
        self.config = None         # nengo_gui.Config for storing layout
        self.components = None     # list of Components
        self.uid_prefix_counter = None # used for generating uids for components
        self.component_uids = None  # mapping from Components to text

        self.config_save_needed = False
        self.config_save_time = None   # time of last config file save
        self.config_save_period = 2.0  # minimum time between saves

        self.lock = threading.Lock()

        # use the default filename if none is given
        if filename is None:
            self.filename = gui.filename
        else:
            try:
                self.filename = os.path.relpath(filename)
            except ValueError:
                # happens on Windows if filename is on a different
                # drive than the current directory
                self.filename = filename

        # determine the .cfg filename
        if gui.filename_cfg is None:
            self.filename_cfg = self.filename + '.cfg'
        else:
            self.filename_cfg = gui.filename_cfg

        if reset_cfg:
            self.clear_config()

        self.load()

        self.net_graph = self.get_component(nengo_gui.components.NetGraph)
        self.ace_editor = self.get_component(nengo_gui.components.AceEditor)

        # build and run the model in a separate thread
        t = threading.Thread(target=self.runner)
        t.daemon = True
        t.start()

    @property
    def sim(self):
        return self._sim

    @sim.setter
    def sim(self, value):
        if hasattr(self._sim, 'close'):
            self._sim.close()
        self._sim = value

    def get_component(self, component_class):
        for c in self.components:
            if isinstance(c, component_class):
                return c
        return None

    def clear_config(self):
        if os.path.isfile(self.filename_cfg):
            os.remove(self.filename_cfg)

    def load(self):
        """Load the model and initialize everything"""
        if self.filename == self.gui.filename:
            # if we're on the default filenaem, just load it from the GUI
            self.model = self.gui.model
            if self.gui.locals is None:
                self.locals = None
            else:
                self.locals = self.gui.locals.copy()
        else:
            self.model = None
            self.locals = None

        # if we still don't have a locals dictionary, then run the script
        if self.locals is None:
            try:
                with open(self.filename) as f:
                    code = f.read()
            except IOError:
                code = ''

            self.execute(code)

        if self.model is None:
            self.model = nengo.Network()
            self.locals['model'] = self.model

        # figure out good names for objects
        self.name_finder = nengo_gui.NameFinder(self.locals, self.model)
        self.default_labels = self.name_finder.known_name

        # load the .cfg file
        self.config = self.load_config()
        self.config_save_needed = False
        self.config_save_time = None   # time of last config file save

        self.uid_prefix_counter = {}

        self.create_components()

    def create_components(self):
        """Generate the actual Components from the Templates"""
        #TODO: change the name of this
        self.components = []
        self.component_uids = {}
        for k, v in self.locals.items():
            if isinstance(v, nengo_gui.components.Component):
                self.component_uids[v] = k
                self.gui.component_uids[id(v)] = v
                self.components.append(v)

        # this ensures NetGraph, AceEditor, and SimControl are first
        self.components.sort(key=lambda x: x.component_order)

    def add_component(self, component):
        """Add a new Component to an existing Page."""
        self.gui.component_uids[id(component)] = component
        uid = self.get_uid(component)
        component.attach(self, self.config[component], uid=uid)
        self.components.append(component)

    def execute(self, code):
        """Run the given code to generate self.model and self.locals.

        The code will be stored in self.code, any output to stdout will
        be a string as self.stdout, and any error will be in self.error.
        """
        locals = {}
        locals['nengo_gui'] = nengo_gui
        locals['__file__'] = self.filename

        self.code = code
        self.error = None
        self.stdout = ''

        exec_env = nengo_gui.exec_env.ExecutionEnvironment(self.filename)
        try:
            with exec_env:
                exec(code, locals)
        except nengo_gui.exec_env.StartedSimulatorException:
            line = nengo_gui.exec_env.determine_line_number()
            exec_env.stdout.write('Warning: Simulators cannot be manually'
                                  ' run inside nengo_gui (line %d)\n' % line)
        except nengo_gui.exec_env.StartedGUIException:
            line = nengo_gui.exec_env.determine_line_number()
            exec_env.stdout.write('Warning: nengo_gui cannot be run inside'
                                  ' nengo_gui (line %d)\n' % line)
        except:
            line = nengo_gui.exec_env.determine_line_number()
            self.error = dict(trace=traceback.format_exc(), line=line)
        self.stdout = exec_env.stdout.getvalue()

        # make sure we've defined a nengo.Network
        model = locals.get('model', None)
        if not isinstance(model, nengo.Network):
            if self.error is None:
                line = len(code.split('\n'))
                self.error = dict(trace='must declare a nengo.Network '
                                  'called "model"', line=line)
            model = None

        self.model = model
        self.locals = locals
        if self.error is None:
            self.last_good_locals = locals

    def load_config(self):
        """Load the .cfg file"""
        config = nengo_gui.config.Config()
        self.locals['nengo_gui'] = nengo_gui
        self.locals['_viz_config'] = config
        fname = self.filename_cfg
        if os.path.exists(fname):
            with open(fname) as f:
                config_code = f.readlines()
            for line in config_code:
                try:
                    exec(line, self.locals)
                except Exception:
                    if self.gui.interactive:
                        logging.debug('error parsing config: %s', line)

        # make sure the required Components exist
        if '_viz_sim_control' not in self.locals:
            c = nengo_gui.components.SimControl()
            self.locals['_viz_sim_control'] = c
        if '_viz_net_graph' not in self.locals:
            c = nengo_gui.components.NetGraph()
            self.locals['_viz_net_graph'] = c
        if '_viz_ace_editor' not in self.locals:
            c = nengo_gui.components.AceEditor()
            self.locals['_viz_ace_editor'] = c

        if self.model is not None:
            if config[self.model].pos is None:
                config[self.model].pos = (0, 0)
            if config[self.model].size is None:
                config[self.model].size = (1.0, 1.0)

        for k, v in self.locals.items():
            if isinstance(v, nengo_gui.components.Component):
                self.default_labels[v] = k
                v.attach(page=self, config=config[v], uid=k)

        return config

    def save_config(self, lazy=False, force=False):
        """Write the .cfg file to disk.

        Parameters
        ----------
        lazy : bool
            If True, then only save if it has been more than config_save_time
            since the last save and if config_save_needed
        force : bool
            If True, then always save right now
        """
        if not force and not self.config_save_needed:
            return

        now_time = time.time()
        if not force and lazy and self.config_save_time is not None:
            if (now_time - self.config_save_time) < self.config_save_period:
                return

        with self.lock:
            self.config_save_time = now_time
            self.config_save_needed = False
            try:
                with open(self.filename_cfg, 'w') as f:
                    f.write(self.config.dumps(uids=self.default_labels))
            except IOError:
                print("Could not save %s; permission denied" %
                      self.filename_cfg)

    def modified_config(self):
        """Set a flag that the config file should be saved."""
        self.config_save_needed = True

    def create_javascript(self):
        """Generate the javascript for the current model and layout."""
        if self.filename is not None:
            fn = json.dumps(self.filename)
            webpage_title_js = ';document.title = %s;' % fn
        else:
            webpage_title_js = ''

        assert isinstance(self.components[0], nengo_gui.components.SimControl)

        component_js = '\n'.join([c.javascript() for c in self.components])
        component_js += webpage_title_js
        if not self.gui.allow_file_change:
            component_js += "$('#Open_file_button').addClass('deactivated');"
        return component_js

    def get_label(self, obj, default_labels=None):
        """Return a readable label for an object.

        If the object has a .label set, this will be used.  Otherwise, it
        uses default_labels, which thanks to the NameFinder will be legal
        Python code for referring to the object given the current locals()
        dictionary ("model.ensembles[1]" or "ens" or "model.buffer.state".
        If it has to use default_labels, it will only use the last part of the
        label (after the last ".").  This avoids redundancy in nested displays.
        """
        if default_labels is None:
            default_labels = self.default_labels
        label = obj.label
        if label is None:
            label = default_labels.get(obj, None)
            # We should never ask for the label for something that can't be
            # found in the default_labels.  If this does happen, something
            # has gone wrong.  Note that this was often a symptom of the
            # dreaded 'pop' bug that causes hassles during the summer school.
            # Hopefully the reorganization of the code into Page and GUI
            # (from Viz and VizSim) has dealt with this problem.
            assert label is not None
            if '.' in label:
                label = label.rsplit('.', 1)[1]
        if label is None:
            label = repr(obj)
        return label

    def get_uid(self, obj, default_labels=None):
        """Return a unique identifier for an object.

        This should be the value given by the NameFinder.
        """
        if default_labels is None:
            default_labels = self.default_labels
        uid = default_labels.get(obj, None)
        if uid is None:
            # TODO: do we ever need to fall back on this case?  This should
            # only happen if something goes wrong.
            uid = repr(obj)
        return uid

    def finish(self):
        """Shut down this page."""
        if not self.finished:
            self.finished = True
            self.gui.remove_page(self)

    def generate_uid(self, obj, prefix):
        """Make a new unique identifier for an object.

        This is used for new Components being created (so they can have
        a unique identifier in the .cfg file).
        """
        index = self.uid_prefix_counter.get(prefix, 0)
        uid = '%s%d' % (prefix, index)
        while uid in self.locals:
            index += 1
            uid = '%s%d' % (prefix, index)
        self.uid_prefix_counter[prefix] = index + 1

        self.locals[uid] = obj
        self.default_labels[obj] = uid

    def remove_uid(self, uid):
        """Remove a generated uid (for when a component is removed)."""
        if uid in self.locals:
            obj = self.locals[uid]
            del self.locals[uid]
            del self.default_labels[obj]
        else:
            print('WARNING: remove_uid called on unknown uid: %s' % uid)

    def remove_component(self, component):
        """Remove a component from the layout."""
        del self.gui.component_uids[id(component)]
        self.remove_uid(component.uid)
        self.components.remove(component)

    def config_change(self, component, new_cfg, old_cfg):
        act = nengo_gui.user_action.ConfigAction(self,
                                                       component=component,
                                                       new_cfg=new_cfg,
                                                       old_cfg=old_cfg)
        self.undo_stack.append([act])

    def remove_graph(self, component):
        act = nengo_gui.user_action.RemoveGraph(self.net_graph,
                                                      component)
        self.undo_stack.append([act])

    def build(self):
        """Build the model."""
        # use the lock to make sure only one Simulator is building at a time
        # TODO: should there be a master lock in the GUI?
        with self.lock:
            self.building = True

            # modify the model for the various Components
            for c in self.components:
                c.add_nengo_objects(self)

            # determine the backend to use
            backend = importlib.import_module(self.backend)
            # if only one Simulator is allowed at a time, finish the old one
            old_sim = Page.singleton_sims.get(self.backend, None)
            if old_sim is not None and old_sim is not self:
                old_sim.sim = None
                old_sim.finished = True

            exec_env = nengo_gui.exec_env.ExecutionEnvironment(self.filename,
                                                               allow_sim=True)
            # build the simulation
            try:
                with exec_env:
                    self.sim = backend.Simulator(self.model)
            except:
                line = nengo_gui.exec_env.determine_line_number()
                self.error = dict(trace=traceback.format_exc(), line=line)

            self.stdout += exec_env.stdout.getvalue()

            if self.sim is not None:
                if self.backend in Page.singleton_sims:
                    Page.singleton_sims[self.backend] = self

            # remove the temporary components added for visualization
            for c in self.components:
                c.remove_nengo_objects(self)
            # TODO: add checks to make sure everything's been removed

            self.building = False
            self.rebuild = False

    def runner(self):
        """Separate thread for running the simulation itself."""
        # run the simulation
        while not self.finished:
            if self.sim is None:
                time.sleep(0.01)
            else:
                try:
                    if hasattr(self.sim, 'max_steps'):
                        # this is only for the nengo_spinnaker simulation
                        self.sim.run_steps(self.sim.max_steps)
                    else:
                        self.sim.step()
                except socket.error:  # if another thread closes the sim
                    pass
                except:
                    line = nengo_gui.exec_env.determine_line_number()
                    self.error = dict(trace=traceback.format_exc(), line=line)
                    self.sim = None

            if self.rebuild:
                self.build()



# Creating Custom Displays

# This demonstrates using the special _nengo_html_ variable to create
# custom visualizations.  In general, this lets you take values from the model
# and use them to make HTML that displays something useful for your situation.
#
# The basic technique is that if you define a function for a Node, in addition
# to returning a value, you can also set the _nengo_html_ parameter on that
# function.  You can set it to any HTML you want, and that HTML will be
# displayed in a custom HTML graph (available by right-clicking on the Node).

# The first example is a timer function that says "Ready..." for the
# first second of the simulation, then says "Set..." for the next second,
# and then says "Go!".  This shows how you can change the HTML based on the
# time of the simulation.
#
# The second example shows that the custom HTML can also be based on the values
# represented by neurons.  Here, we make a Node that reads in a value that is
# treated as an amount.  If the value is above 0.5, the HTML says "large", 
# if the value is below -0.5, it says "small", and otherwise it says "medium".
#
# Finally, the third example uses SVG and trigonometry to create a simple
# visualization of a two-joint arm.  The two values represented by the
# Ensemble are the two joint angles (the shoulder and the elbow).  From these
# values, the position of the elbow and hand are computed, and lines are
# drawn to show the arm.

import nengo
import numpy as np

model = nengo.Network()
with model:
    
    # Example 1: a timer
    def timer_function(t):
        if t < 1.0:
            timer_function._nengo_html_ = '<h1>Ready...</h1>'
            return 0
        elif t < 2.0:
            timer_function._nengo_html_ = '<h1>Set...</h1>'
            return 0
        else:
            timer_function._nengo_html_ = '<h1>Go!</h1>'
            return 1
    timer = nengo.Node(timer_function)
            
            
    # Example 2: displaying a value        
    def amount_function(t, x):
        if x < -0.5:
            amount_function._nengo_html_ = '<h2>small</h2>'
        elif -0.5 < x < 0.5:
            amount_function._nengo_html_ = '<h2>medium</h2>'
        else:
            amount_function._nengo_html_ = '<h2>large</h2>'
    stim_amount = nengo.Node(0)
    amount = nengo.Ensemble(n_neurons=100, dimensions=1)
    display_amount = nengo.Node(amount_function, size_in=1)
    nengo.Connection(stim_amount, amount)
    nengo.Connection(amount, display_amount)
            
            
    # Example 3: a two-joint arm    
    def arm_function(t, angles):
        len0 = 50
        len1 = 30
        x1 = 50
        y1 = 100
        x2 = x1 + len0 * np.sin(angles[0])
        y2 = y1 - len0 * np.cos(angles[0])
        x3 = x2 + len1 * np.sin(angles[0] + angles[1])
        y3 = y2 - len1 * np.cos(angles[0] + angles[1])
        arm_function._nengo_html_ = '''
        <svg width="100%" height="100%" viewbox="0 0 100 100">
            <line x1="{x1}" y1="{y1}" x2="{x2}" y2="{y2}" style="stroke:black"/>
            <line x1="{x2}" y1="{y2}" x2="{x3}" y2="{y3}" style="stroke:black"/>
        </svg>
        '''.format(**locals())
    stim_angles = nengo.Node([0.3, 0.3])
    angles = nengo.Ensemble(n_neurons=200, dimensions=2)
    arm = nengo.Node(arm_function, size_in=2)
    nengo.Connection(stim_angles, angles)
    nengo.Connection(angles, arm)
    


# Nengo Example: Many neurons
#
# This demo shows how to construct and manipulate a population of neurons.
#
# These are 100 leaky integrate-and-fire (LIF) neurons. The neuron tuning
# properties have been randomly selected.
#
# The input is a sine wave to show the effects of increasing or decreasing
# input. As a population, these neurons do a good job of representing a
# single scalar value.

import nengo

model = nengo.Network()
with model:
    # Our ensemble consists of 100 leaky integrate-and-fire neurons,
    # representing a one-dimensional signal
    a = nengo.Ensemble(n_neurons=100, dimensions=1)

    stim = nengo.Node([0])

    # Connect the input to the population
    nengo.Connection(stim, a)



import json
import mimetypes
import os
import os.path
import pkgutil
import time
import traceback

try:
    from urllib import unquote
except ImportError:
    from urllib.parse import unquote

import nengo_gui.swi as swi
import nengo_gui


class Server(swi.SimpleWebInterface):
    """Web server interface to nengo_gui"""

    def swi_browse(self, dir):
        if self.user is None: return
        r = ['<ul class="jqueryFileTree" style="display: none;">']
        d = unquote(dir)
        ex_tag = '//examples//'
        ex_html = '<em>built-in examples</em>'
        if d == '.':
            r.append('<li class="directory collapsed examples_dir">'
                     '<a href="#" rel="%s">%s</a></li>' % (ex_tag, ex_html))
            path = '.'
        elif d.startswith(ex_tag):
            path = os.path.join(nengo_gui.__path__[0],
                                'examples', d[len(ex_tag):])
        else:
            path = os.path.join('.', d)

        for f in sorted(os.listdir(path)):
            ff = os.path.join(path, f)
            if os.path.isdir(os.path.join(path, f)):
                r.append('<li class="directory collapsed">'
                         '<a href="#" rel="%s/">%s</a></li>' % (ff,f))
            else:
                e = os.path.splitext(f)[1][1:] # get .ext and remove dot
                if e == 'py':
                    r.append('<li class="file ext_%s">'
                             '<a href="#" rel="%s">%s</a></li>' % (e,ff,f))
        r.append('</ul>')
        return ''.join(r)

    def swi_static(self, *path):
        """Handles http://host:port/static/* by returning pkg data"""
        fn = os.path.join('static', *path)
        mimetype, encoding = mimetypes.guess_type(fn)
        data = pkgutil.get_data('nengo_gui', fn)
        return (mimetype, data)

    def swi_favicon_ico(self):
        icon = pkgutil.get_data('nengo_gui', 'static/favicon.ico')
        return ('image/ico', icon)

    def create_login_form(self):
        if self.attempted_login:
            message = 'Invalid password. Try again.'
        else:
            message = 'Enter the password:'
        return """<form action="/" method=GET>%s<br>
            <input type=hidden name=swi_id value=''>
            <input type=password name=swi_pwd>
            <input type=submit value="Log In">
            </form>""" % message

    def swi(self, filename=None, reset=None):
        """Handles http://host:port/ by giving the main page"""
        if self.user is None:
            return self.create_login_form()

        reset_cfg = reset == 'True'

        page = self.server.gui.create_page(filename, reset_cfg=reset_cfg)

        # read the template for the main page
        html = pkgutil.get_data('nengo_gui', 'templates/page.html')
        if isinstance(html, bytes):
            html = html.decode("utf-8")

        # fill in the javascript needed and return the complete page
        components = page.create_javascript()
        return html % dict(components=components)

    def swi_shutdown(self, *path):
        self.stop()
        return "Shutting down..."

    def ws_viz_component(self, client, uid):
        """Handles ws://host:port/viz_component with a websocket"""
        # figure out what component is being connected to

        gui = self.server.gui

        component = gui.component_uids[int(uid)]
        while True:
            try:
                if gui.finished:
                    break
                if component.replace_with is not None:
                    component.finish()
                    component = component.replace_with
                # read all data coming from the component
                msg = client.read()
                while msg is not None:
                    if msg.startswith('config:'):
                        cfg = json.loads(msg[7:])
                        old_cfg = {}
                        for k in component.config_defaults.keys():
                            v = getattr(
                                component.page.config[component], k)
                            old_cfg[k] = v
                        if not(cfg == old_cfg):
                            # Register config change to the undo stack
                            component.page.config_change(
                                component, cfg, old_cfg)
                        for k, v in cfg.items():
                            setattr(
                                component.page.config[component],
                                k, v)
                        component.page.modified_config()
                    elif msg.startswith('remove'):
                        if msg != 'remove_undo':
                            # Register graph removal to the undo stack
                            component.page.remove_graph(component)
                        component.page.remove_component(component)
                        component.page.modified_config()
                        return
                    else:
                        try:
                            component.message(msg)
                        except:
                            print('Error processing: "%s"' % msg)
                            traceback.print_exc()
                    msg = client.read()
                # send data to the component
                component.update_client(client)
                component.page.save_config(lazy=True)
                time.sleep(0.01)
            except swi.SocketClosedError:
                # This error means the server has shut down
                component.page.save_config(lazy=False)  # Stop nicely
                break
            except:
                traceback.print_exc()

        # After hot loop
        component.finish()

        if isinstance(component, nengo_gui.components.SimControl):
            component.page.sim = None

        if client.remote_close:
            # wait a moment before checking if the server should be stopped
            time.sleep(2)

            # if there are no Pages left, stop the server
            if isinstance(component, nengo_gui.components.SimControl):
                if gui.count_pages() == 0:
                    if gui.interactive:
                        print("No connections remaining to the nengo_gui "
                              "server.")
                    self.server.shutdown()

    def log_message(self, format, *args):
        # suppress all the log messages
        pass



import numpy as np
import nengo

from nengo_gui.components.component import Component
from nengo_gui.components.spa_plot import SpaPlot


class SpaSimilarity(SpaPlot):

    config_defaults = dict(max_value=1,
                           min_value=-1,
                           show_pairs=False,
                           **Component.config_defaults)

    def __init__(self, obj, **kwargs):
        super(SpaSimilarity, self).__init__(obj, **kwargs)

        self.old_vocab_length = len(self.vocab_out.keys)
        self.old_pairs_length = 0
        self.labels = self.vocab_out.keys
        self.previous_pairs = False

        # Nengo objects for data collection
        self.node = None
        self.conn = None

    def add_nengo_objects(self, page):
        with page.model:
            output = self.obj.outputs[self.target][0]
            self.node = nengo.Node(self.gather_data,
                                   size_in=self.vocab_out.dimensions)
            self.conn = nengo.Connection(output, self.node, synapse=0.01)

    def remove_nengo_objects(self, page):
        """Undo the changes made by add_nengo_objects."""
        page.model.connections.remove(self.conn)
        page.model.nodes.remove(self.node)

    def gather_data(self, t, x):
        vocab = self.vocab_out

        if self.old_vocab_length != len(vocab.keys):
            self.update_legend(vocab)

        # get the similarity and send it
        key_similarity = np.dot(vocab.vectors, x)
        simi_list = ['{:.2f}'.format(simi) for simi in key_similarity]

        if self.config.show_pairs:

            # briefly there can be no pairs, so catch the error
            try:
                pair_similarity = np.dot(vocab.vector_pairs, x)
                simi_list += ['{:.2f}'.format(simi) for simi in pair_similarity]
            except TypeError:
                pass

        if(simi_list != []):
            self.data.append(  '["data_msg", %g, %s]'
                             %( t, ",".join(simi_list) )  )

    def update_legend(self, vocab):
        # pass all the missing keys
        legend_update = []
        legend_update += (vocab.keys[self.old_vocab_length:])
        self.old_vocab_length = len(vocab.keys)
        # and all the missing pairs if we're showing pairs
        if self.config.show_pairs:
        # briefly there can be no pairs, so catch the error
            try:
                legend_update += vocab.key_pairs[self.old_pairs_length:]
                self.old_pairs_length = len(vocab.key_pairs)
            except TypeError:
                pass

        self.data.append('["update_legend", "%s"]'
                         %('","'.join(legend_update)))


    def javascript(self):
        """Generate the javascript that will create the client-side object"""
        info = dict(uid=id(self), label=self.label, n_lines=len(self.labels),
                    synapse=0, min_value=-1.5, max_value=1.5,
                    pointer_labels=self.labels)
        json = self.javascript_config(info)
        return 'new Nengo.SpaSimilarity(main, sim, %s);' % json

    def message(self, msg):
        """Message receive function for show_pairs toggling and reset"""
        vocab = self.vocab_out
        # Send the new labels
        if self.config.show_pairs:
            vocab.include_pairs = True
            self.data.append(
                '["reset_legend_and_data", "%s"]' %(
                    '","'.join(vocab.keys + vocab.key_pairs)))
            # if we're starting to show pairs, track pair length
            self.old_pairs_length = len(vocab.key_pairs)
        else:
            vocab.include_pairs = False
            self.data.append('["reset_legend_and_data", "%s"]'
                             %('","'.join(vocab.keys)))



import argparse

import nengo_gui
import nengo_gui.gui


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-p', '--password', dest='password', metavar='PASS',
        help='password for remote access')
    parser.add_argument(
        '-P', '--port', dest='port', metavar='PORT',
        default=8080, type=int, help='port to run server on')
    parser.add_argument(
        'filename', nargs='?', type=str, help='initial file to load')
    parser.add_argument(
        '--debug', action='store_true', help='turn on debug logging')
    parser.add_argument(
        '-b', '--backend', metavar='BACKEND',
        default='nengo', type=str, help='default backend to use')
    parser.add_argument('--browser', dest='browser', action='store_true')
    parser.add_argument('--no-browser', dest='browser', action='store_false')
    parser.set_defaults(browser=True)
    args = parser.parse_args()

    if args.debug:
        import logging
        logging.basicConfig(level=logging.DEBUG)

    s = nengo_gui.gui.GUI(filename=args.filename,
                                       backend=args.backend)

    s.start(port=args.port, password=args.password, browser=args.browser)

if __name__ == '__main__':
    main()



import pytest
import nengo
from nengo import spa

from nengo_gui.components.value import Value

def test_output_detect():
    model = spa.SPA()

    with model:

        # Find the output for ens
        ens = nengo.Ensemble(10, 2)
        val = Value(ens)
        assert val.n_lines == 2
        assert val.output == ens

        # Find output for node
        node = nengo.Node([0, 0, 0])
        val = Value(node)
        assert val.n_lines == 3
        assert val.output == node

        # Find output for network
        ea = nengo.networks.EnsembleArray(10, 4)
        val = Value(ea)
        assert val.n_lines == 4
        assert val.output == ea.output

        # Find output for SPA module
        model.comp = spa.Compare(4)
        val = Value(model.comp)
        assert val.n_lines == 1
        assert val.output == model.comp.output



# Tutorial 9: Multiplication

# Now that we can combine information, we can use this to compute more
# complex functions.  For example, to multiply two numbers together, we
# first make a combined Ensemble as in the previous tutorial, and then we
# compute the pruduct of the two numbers by multiplying them together when
# we make a Connection out of that combined Ensemble.

# Notice that we had to increase the radius of the combined Ensemble to 1.5.
# That is because it is representing two values, each of which can be in the
# range -1 to 1.  In order to make sure the Ensemble is good at representing
# both values, we need to make sure the radius is large enough to include the
# point [1, 1].  This requires a radius of sqrt(2), or around 1.5.

import nengo

model = nengo.Network()
with model:
    stim_a = nengo.Node(0)
    a = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim_a, a)
    
    stim_b = nengo.Node(0)
    b = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim_b, b)
    
    c = nengo.Ensemble(n_neurons=200, dimensions=2, radius=1.5)
    nengo.Connection(a, c[0])
    nengo.Connection(b, c[1])
    
    d = nengo.Ensemble(n_neurons=50, dimensions=1)
    
    def multiply(x):
        return x[0] * x[1]
    nengo.Connection(c, d, function=multiply)



import json

class Component(object):
    """Abstract handler for a particular Component of the user interface.

    Each part of the user interface has part of the code on the server-side
    (in Python) and a part on the client-side (in Javascript).  These two sides
    communicate via WebSockets, and the server-side is always a subclass of
    Component.

    Each Component can be configured via the nengo.Config system.  Components
    can add required nengo objects into the model to allow them to gather
    the required data from the running model.  Communication from server to 
    client is done via Component.update_client(), which is called regularly
    by the Server.ws_viz_component handler.  Communication from client to
    server is via Component.message().
    """

    # The parameters that will be stored in the .cfg file for this Component
    # Subclasses should override this as needed.
    config_defaults = dict(x=0, y=0, width=100, height=100, label_visible=True)

    def __init__(self, component_order=0):
        # when generating Javascript for all the Components in a Page, they
        # will be sorted by component_order.  This way some Components can
        # be defined before others.
        self.component_order = component_order

        # If we have reloaded the model (while typing in the editor), we need
        # to swap out the old Component with the new one.
        self.replace_with = None

        # If we have been swapped out, keep track of the id of the original
        # component, since that's the identifier needed to refer to it on
        # the client side
        self.original_id = id(self)

    def attach(self, page, config, uid):
        """Connect the Component to a Page."""
        self.config = config  # the nengo.Config[component] for this component
        self.page = page      # the Page this component is in
        self.uid = uid        # The Python string referencing this component

    def update_client(self, client):
        """Send any required information to the client.

        This method is called regularly by Server.ws_viz_component().  You
        send text data to the client-side via a WebSocket as follows:
            client.write(data)
        You send binary data as:
            client.write(data, binary=True)
        """
        pass

    def message(self, msg):
        """Receive data from the client.

        Any data sent by the client ove the WebSocket will be passed into
        this method.
        """
        print('unhandled message', msg)

    def finish(self):
        """Close this Component"""
        pass

    def add_nengo_objects(self, page):
        """Add or modify the nengo model before build.

        Components may need to modify the underlying nengo.Network by adding
        Nodes and Connections or modifying the structure in other ways.
        This method will be called for all Components just before the build
        phase.
        """
        pass

    def remove_nengo_objects(self, page):
        """Undo the effects of add_nengo_objects.

        After the build is complete, remove the changes to the nengo.Network
        so that it is all set to be built again in the future.
        """
        pass

    def javascript_config(self, cfg):
        """Convert the nengo.Config information into javascript.
        
        This is needed so we can send that config information to the client.
        """
        for attr in self.config._clsparams.params:
            cfg[attr] = getattr(self.config, attr)
        return json.dumps(cfg)

    def code_python(self, uids):
        """Generate Python code for this Component.

        This is used in the .cfg file to generate a valid Python expression
        that re-creates this Component.  

        The input uids is a dictionary from Python objects to strings that
        refer to those Python objects (the reverse of the locals() dictionary)
        """
        args = self.code_python_args(uids)
        name = self.__class__.__name__
        return 'nengo_gui.components.%s(%s)' % (name, ','.join(args))

    def code_python_args(self, uids):
        """Return a list of strings giving the constructor arguments.

        This is used by code_python to re-create the Python string that
        generated this Component, so it can be saved in the .cfg file.

        The input uids is a dictionary from Python objects to strings that
        refer to those Python objects (the reverse of the locals() dictionary)
        """
        return []




import numpy as np
import struct
import collections

from nengo_gui.components.component import Component

class Slider(Component):
    config_defaults = dict(max_value=1, min_value=-1, 
                           **Component.config_defaults)

    def __init__(self, node):
        super(Slider, self).__init__()
        self.node = node
        self.base_output = node.output
        self.override = [None] * node.size_out
        self.last_time = None
        self.value = np.zeros(node.size_out)
        self.start_value = np.zeros(node.size_out, dtype=float)
        self.struct = struct.Struct('<%df' % (1 + node.size_out))
        self.data = collections.deque()
        if not callable(self.base_output):
            self.start_value[:] = self.base_output

    def attach(self, page, config, uid):
        super(Slider, self).attach(page, config, uid)
        self.label = page.get_label(self.node)

    def add_nengo_objects(self, page):
        self.node.output = self.override_output

    def remove_nengo_objects(self, page):
        self.node.output = self.base_output

    def override_output(self, t, *args):
        if self.last_time is None or t < self.last_time:
            self.override = [None] * self.node.size_out
        self.last_time = t
        if callable(self.base_output):
            self.value[:] = self.base_output(t, *args)
            self.data.append(self.struct.pack(t, *self.value))
        else:
            self.value[:] = self.base_output

        for i, v in enumerate(self.override):
            if v is not None:
                self.value[i] = v
        return self.value

    def javascript(self):
        info = dict(uid=id(self), n_sliders=len(self.override),
                    label=self.label,
                    start_value=[float(x) for x in self.start_value])
        json = self.javascript_config(info)
        return 'new Nengo.Slider(main, sim, %s);' % json


    def update_client(self, client):
        while len(self.data) > 0:
            data = self.data.popleft()
            client.write(data, binary=True)

    def message(self, msg):
        index, value = msg.split(',')
        index = int(index)
        if value == 'reset':
            self.override[index] = None
        else:
            value = float(value)
            self.override[index] = value

    def code_python_args(self, uids):
        return [uids[self.node]]



import nengo
from nengo_gui.grandalf.graphs import Vertex, Edge, Graph
from nengo_gui.grandalf.layouts import VertexViewer, SugiyamaLayout


class Layout(object):
    """Generates layouts for nengo Networks"""
    def __init__(self, model):
        self.model = model

        # dictionary to keep track of parents of items in Network
        self.parents = {}

        # subnetworks that have not yet been examined for parents
        self.unexamined_networks = [model]

    def find_parent(self, obj):
        """Return the parent of an object in the model.

        The layout system needs to know the parents of items so that it can
        handle a Connection into a deeply nested subnetwork (that Connection
        should be treated as a Connection to the Network that is a direct
        child of the Network we are currently laying out).  But, we don't
        want to do a complete search of the entire graph.  So, instead we
        do an incremental breadth-first search until we find the component
        we are looking for.
        """
        if obj is self.model:
            # the top Network does not have a parent
            return None
        parent = self.parents.get(obj, None)
        while parent is None:
            if len(self.unexamined_networks) == 0:
                # there are no networks left we haven't looked into
                # this should not happen in a valid nengo.Network
                print("could not find parent of", obj)
                return None
            # grab the next network we haven't looked into
            net = self.unexamined_networks.pop(0)
            # identify all its children
            for n in net.nodes:
                self.parents[n] = net
            for e in net.ensembles:
                self.parents[e] = net
            for n in net.networks:
                self.parents[n] = net
                # add child networks into the list to be searched
                self.unexamined_networks.append(n)
            parent = self.parents.get(obj, None)
        return parent

    def compute_bounds(self, core):
        """Determine the min/max x/y values in the graph_core"""
        minx = None
        maxx = None
        miny = None
        maxy = None

        for v in core.V():
            x0 = v.view.xy[0] - v.view.w / 2.0
            x1 = v.view.xy[0] + v.view.w / 2.0
            y0 = v.view.xy[1] - v.view.h / 2.0
            y1 = v.view.xy[1] + v.view.h / 2.0
            if minx is None or x0 < minx:
                minx = x0
            if maxx is None or x1 > maxx:
                maxx = x1
            if miny is None or y0 < miny:
                miny = y0
            if maxy is None or y1 > maxy:
                maxy = y1
        return minx, miny, maxx, maxy

    def make_layout(self, network):
        """Generate a feed-forward layout for this network"""

        # build the graph to pass into grandalf
        # note that grandalf flows from top to bottom, so x and y are switched
        # from what we do in nengo_gui (so the flow is left to right)
        vertices = {}
        for n in network.nodes:
            vertices[n] = Vertex(n)
            # default sizes for nodes: 10x20
            vertices[n].view = VertexViewer(w=8, h=16)
        for e in network.ensembles:
            vertices[e] = Vertex(e)
            # default sizes for ensembles: 10x20
            vertices[e].view = VertexViewer(w=10, h=20)
        for n in network.networks:
            vertices[n] = Vertex(n)
            # default sizes for networks: 40x40
            vertices[n].view = VertexViewer(w=40, h=40)

        # define the connections.  Any connection to a component inside a
        # subnetwork is replaced with a connection to the parent that is a
        # direct child of this network
        edges = {}
        for c in network.connections:
            pre = c.pre_obj
            if isinstance(pre, nengo.ensemble.Neurons):
                pre = pre.ensemble
            while pre not in vertices:
                pre = self.find_parent(pre)
                if pre is None:
                    break
            post = c.post_obj
            if isinstance(post, nengo.ensemble.Neurons):
                post = post.ensemble
            while post not in vertices:
                post = self.find_parent(post)
                if post is None:
                    break

            if pre is None or post is None:
                # the connection does not go to a child of this network,
                # so ignore it.
                print('error processing', c)
            else:
                edges[c] = Edge(vertices[pre], vertices[post], data=c)

        # generate the graph
        graph = Graph(vertices.values(), edges.values())

        # do the layouts.  Note that each graph core is laid out separately,
        # since the layout algorithm requires a connected graph.  (The graph
        # cores are the separate connected graphs in the full network)
        layouts = [SugiyamaLayout(g) for g in graph.C]
        for layout in layouts:
            layout.init_all()
            layout.draw(3)  # do three passes to improve crossings

        # now rescale all the layouts to fit within a (0,0,1,1) bounding box

        bounds = [self.compute_bounds(g) for g in graph.C]

        widths = [b[2] - b[0] for b in bounds]
        heights = [b[3] - b[1] for b in bounds]

        # amount of spacing between cores and the margin around the space
        # This is in graph layout units
        spacing = 5

        # total width of all graphs
        total_width = sum(widths) + spacing * (len(widths) + 1)

        scale_x = 1.0 / total_width

        # starting location in network position units (0 to 1)
        x0 = spacing * scale_x

        pos = {}

        # do the rescaling
        # minx, miny, maxx, maxy are in the space of the core being laid out
        # x0, y0, x1, y1 are in the space of the network positions
        for i, core in enumerate(graph.C):
            scale_y = 1.0 / (heights[i] + spacing * 2)
            x1 = x0 + widths[i] * scale_x
            y0 = spacing * scale_y
            y1 = 1.0 - y0

            minx, miny, maxx, maxy = bounds[i]

            for v in core.V():
                x = x0 + (v.view.xy[0] - minx) * (x1 - x0) / (maxx - minx)
                y = y0 + (v.view.xy[1] - miny) * (y1 - y0) / (maxy - miny)
                w = v.view.w * (x1 - x0) / (maxx - minx)
                h = v.view.h * (y1 - y0) / (maxy - miny)
                pos[v.data] = dict(x=x, y=y, w=w, h=h)

            # place the next core beside this one
            x0 = x1 + spacing * scale_x

        return pos



# The Lorenz chaotic attractor
#
# This example shows the construction of a classic chaotic dynamical system:
# the Lorenz "butterfly" attractor.  The equations are:
#
# dx0/dt = sigma * (x1 - x0)
# dx1/dt = x0 * (rho - x2) - x1
# dx2/dt = x0 * x1 - beta * x2
#
# Since x2 is centered around approximately rho, and since NEF ensembles
# are usually optimized to represent values within a certain radius of the 
# origin, we substitute x2' = x2 - rho, giving these equations:
#
# dx0/dt = sigma * (x1 - x0)
# dx1/dt = - x0 * x2' - x1
# dx2/dt = x0 * x1 - beta * (x2 + rho) - rho
#
# For more information, see 
# http://compneuro.uwaterloo.ca/publications/eliasmith2005b.html
# "Chris Eliasmith. A unified approach to building and controlling 
# spiking attractor networks. Neural computation, 7(6):1276-1314, 2005."

tau = 0.1
sigma = 10
beta = 8.0/3
rho = 28

import nengo

def feedback(x):
    dx0 = -sigma * x[0] + sigma * x[1]
    dx1 = -x[0] * x[2] - x[1]
    dx2 = x[0] * x[1] - beta * (x[2] + rho) - rho

    return [dx0 * tau + x[0],
            dx1 * tau + x[1],
            dx2 * tau + x[2]]

model = nengo.Network(seed=1)
with model:
    state = nengo.Ensemble(2000, 3, radius=30)
    nengo.Connection(state, state, function=feedback, synapse=tau)



import numpy as np
import nengo

model = nengo.Network()
with model:
    stimulus_A = nengo.Node([1], label='stim A')
    stimulus_B = nengo.Node(lambda t: np.sin(2*np.pi*t))
    ens = nengo.Ensemble(n_neurons=1000, dimensions=2)
    result = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stimulus_A, ens[0])
    nengo.Connection(stimulus_B, ens[1])
    nengo.Connection(ens, result, function=lambda x: x[0] * x[1],
                     synapse=0.01)

if __name__ == '__main__':
    import nengo_gui
    nengo_gui.Viz(__file__).start()



# Tutorial 11: Memory

# Nengo models can also store information over time.  To do this, we simply
# make a Connection from an Ensemble back to itself.  That is, we form a
# Connection that will feed a value into an Ensemble that is the same value
# that the Ensemble is currently representing.  This means we can store data
# over time.

# To use such a system, connect into it with another Ensemble.  If that input
# is zero, then the stored value should stay the same as it currently is.
# If the input is positive, the stored value should increase.  If the input is
# negative, it should decrease.

# Notice that the input Connection has "transform=0.1".  That is to control
# how strongly the input affects the stored value.  If you make the transform
# larger, it will change more quickly.

# Also notice that the recurrent Connection from b back to itself has
# synapse=0.1.  This longer time constant makes the memory more stable, and
# is also commonly found in the real brain for recurrent connections.

# Mathematical Note: In the case where the input transform is exactly equal to
# the recurrent synapse (as it is here), it turns out that the resulting system
# should compute the mathematical integral of the input.

import nengo

model = nengo.Network()
with model:
    stim_a = nengo.Node(0)
    a = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim_a, a)
    
    b = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(b, b, synapse=0.1)
    nengo.Connection(a, b, transform=0.1)
    



import nengo
import numpy as np
import struct

from nengo_gui.components.component import Component

class SpikeGrid(Component):
    def __init__(self, obj, n_neurons=None):
        super(SpikeGrid, self).__init__()
        self.obj = obj
        self.data = []
        self.max_neurons = self.obj.neurons.size_out
        if n_neurons is None:
            n_neurons = self.max_neurons
        self.n_neurons = n_neurons
        self.pixels_x = np.ceil(np.sqrt(self.n_neurons))
        self.pixels_y = np.ceil(float(self.n_neurons) / self.pixels_x)
        self.n_pixels = self.pixels_x * self.pixels_y
        self.struct = struct.Struct('<f%dB' % (self.n_pixels))
        self.max_value = 1.0

    def attach(self, page, config, uid):
        super(SpikeGrid, self).attach(page, config, uid)
        self.label = page.get_label(self.obj)

    def add_nengo_objects(self, page):
        with page.model:
            self.node = nengo.Node(self.gather_data,
                                   size_in=self.obj.neurons.size_out)
            self.conn = nengo.Connection(self.obj.neurons,
                                         self.node, synapse=0.01)

    def remove_nengo_objects(self, page):
        page.model.connections.remove(self.conn)
        page.model.nodes.remove(self.node)

    def gather_data(self, t, x):
        self.max_value = max(self.max_value, np.max(x))
        if len(x) > self.n_neurons:
            x = x[:self.n_neurons]
        y = np.zeros(self.n_pixels, dtype=np.uint8)
        if self.max_value > 0:
            y[:x.size] = x * 255 / self.max_value
        data = self.struct.pack(t, *y)
        self.data.append(data)

    def update_client(self, client):
        length = len(self.data)
        if length > 0:
            item = bytes().join(self.data[:length])
            del self.data[:length]
            try:
                client.write(item, binary=True)
            except:
                # if there is a communication problem, just drop the frames
                # (this usually happens when there is too much data to send)
                pass

    def javascript(self):
        info = dict(uid=id(self), label=self.label,
                    pixels_x=self.pixels_x, pixels_y=self.pixels_y)
        json = self.javascript_config(info)
        return 'new Nengo.Image(main, sim, %s);' % json

    def code_python_args(self, uids):
        args = [uids[self.obj]]
        if self.n_neurons != self.max_neurons:
            args.append('n_neurons=%d' % self.n_neurons)
        return args



import collections

from nengo.spa.module import Module

from nengo_gui.components.component import Component

class SpaPlot(Component):
    """Parent class for pointer.Pointer and
    spa_similarity.SpaSimilarity"""

    def __init__(self, obj, **kwargs):
        super(SpaPlot, self).__init__()
        self.obj = obj
        self.data = collections.deque()
        self.target = kwargs.get('args', 'default')
        self.vocab_out = obj.outputs[self.target][1]

    def attach(self, page, config, uid):
        super(SpaPlot, self).attach(page, config, uid)
        self.label = page.get_label(self.obj)
        self.vocab_out.include_pairs = config.show_pairs

    def update_client(self, client):
        while len(self.data) > 0:
            data = self.data.popleft()
            client.write(data, binary=False)

    def code_python_args(self, uids):
        return [uids[self.obj], 'target=%r' % self.target]

    @staticmethod
    def applicable_targets(obj):
        targets = []
        if isinstance(obj, Module):
            for target_name, (obj, vocab) in obj.outputs.items():
                if vocab is not None:
                    targets.append(target_name)
        return targets


import inspect

import nengo

import nengo_gui.components


class Config(nengo.Config):
    def __init__(self):
        super(Config, self).__init__()
        for cls in [nengo.Ensemble, nengo.Node]:
            self.configures(cls)
            self[cls].set_param('pos', nengo.params.Parameter(None))
            self[cls].set_param('size', nengo.params.Parameter(None))
        self.configures(nengo.Network)
        self[nengo.Network].set_param('pos', nengo.params.Parameter(None))
        self[nengo.Network].set_param('size', nengo.params.Parameter(None))
        self[nengo.Network].set_param('expanded',
                                      nengo.params.Parameter(False))
        self[nengo.Network].set_param('has_layout',
                                      nengo.params.Parameter(False))

        for clsname, cls in inspect.getmembers(nengo_gui.components):
            if inspect.isclass(cls):
                if issubclass(cls, nengo_gui.components.component.Component):
                    if cls != nengo_gui.components.component.Component:
                        self.configures(cls)
                        for k, v in cls.config_defaults.items():
                            self[cls].set_param(k, nengo.params.Parameter(v))

    def dumps(self, uids):
        lines = []
        for obj, uid in sorted(uids.items(), key=lambda x: x[1]):
            if isinstance(obj, (nengo.Ensemble, nengo.Node, nengo.Network)):
                if self[obj].pos is not None:
                    lines.append('_viz_config[%s].pos=%s' % (uid,
                                                             self[obj].pos))
                if self[obj].size is not None:
                    lines.append('_viz_config[%s].size=%s' % (uid,
                                                              self[obj].size))
                if isinstance(obj, nengo.Network):
                    lines.append('_viz_config[%s].expanded=%s'
                                 % (uid, self[obj].expanded))
                    lines.append('_viz_config[%s].has_layout=%s'
                                 % (uid, self[obj].has_layout))
            elif isinstance(obj, nengo_gui.components.component.Component):
                lines.append('%s = %s' % (uid, obj.code_python(uids)))
                for k in obj.config_defaults.keys():
                    v = getattr(self[obj], k)
                    if isinstance(v, bool):
                        val = '%s' % v
                    else:
                        val = '%g' % v
                    lines.append('_viz_config[%s].%s = %s' % (uid, k, val))

        return '\n'.join(lines)



# Tutorial 1: A Single Neuron

# Here we show one single neuron.  The slider on the left adjusts the input
# to that neuron.  The top graph shows the voltage in the neuron.  When that
# voltage is high enough, the neuron "spikes", producing an output (middle
# graph).  That output releases neurotransmitter which is gradually
# reabsorbed.  Given only that neurotransmitter output, it is difficult
# to reconstruct the original input (bottom graph).

# User Interface Tip: You can adjust the amount of time shown on the graphs
# by dragging the left side of the gray bar inside the timeline at the bottom
# of the screen.  Try reducing it down to a smaller size so you can see
# the individual spikes better.

import nengo

model = nengo.Network()
with model:
    stim = nengo.Node(0)
    ens = nengo.Ensemble(n_neurons=1, dimensions=1, seed=5)
    nengo.Connection(stim, ens)

    # include these lines if the simulation is too fast
    import time
    slowdown = nengo.Node(lambda t: time.sleep(0.001))



import struct
import collections

import nengo
import numpy as np

from nengo_gui.components.component import Component


class XYValue(Component):
    config_defaults = dict(max_value=1, min_value=-1, index_x=0, index_y=1,
                           **Component.config_defaults)
    def __init__(self, obj):
        super(XYValue, self).__init__()
        self.obj = obj
        self.data = collections.deque()
        self.n_lines = int(obj.size_out)
        self.struct = struct.Struct('<%df' % (1 + self.n_lines))

    def attach(self, page, config, uid):
        super(XYValue, self).attach(page, config, uid)
        self.label = page.get_label(self.obj)

    def add_nengo_objects(self, page):
        with page.model:
            self.node = nengo.Node(self.gather_data,
                                   size_in=self.obj.size_out)
            self.conn = nengo.Connection(self.obj, self.node, synapse=0.01)

    def remove_nengo_objects(self, page):
        page.model.connections.remove(self.conn)
        page.model.nodes.remove(self.node)

    def gather_data(self, t, x):
        self.data.append(self.struct.pack(t, *x))

    def update_client(self, client):
        while len(self.data) > 0:
            data = self.data.popleft()
            client.write(data, binary=True)

    def javascript(self):
        info = dict(uid=id(self), n_lines=self.n_lines, label=self.label)
        json = self.javascript_config(info)
        return 'new Nengo.XYValue(main, sim, %s);' % json

    def code_python_args(self, uids):
        return [uids[self.obj]]



# Tutorial 0: Welcome to Nengo

# Nengo <http://nengo.ca> is a tool for creating large-scale biologically
# realistic neural models.  It was developed by the Centre for Theoretical
# Neuroscience at the University of Waterloo and the affiliated spin-off
# company Applied Brain Research.  It has been used to create Spaun, the
# first simulated brain that is capable of performing tasks.

# This sequence of tutorials takes you through the various features of Nengo.
#
# You can go to the next tutorial by clicking on the "Open file" icon in the
# top-left of the screen and selecting the next one.
#
# The Nengo interface shows the script that defines the model here on the
# right side of the screen, and a graphical depiction of the model on the left
# side.
#
# If you press the "Play" button (bottom-right), the simulation will start.
# You can move the slider to adjust the input to the neurons and see the
# spiking activity of the neurons change.  You can also see the value that
# can be decoded from that spiking activity change as the neurons try to
# represent whatever value you are inputting.

import nengo

model = nengo.Network()
with model:
    stim = nengo.Node(0)
    ens = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim, ens)
    
    # include these lines if the simulation is too fast
    import time
    slowdown = nengo.Node(lambda t: time.sleep(0.001))



import time
import os
import traceback
import collections
import threading

import nengo
from nengo import spa
import json

from nengo_gui.components.component import Component
from nengo_gui.components.value import Value
from nengo_gui.modal_js import infomodal
import nengo_gui.user_action
import nengo_gui.layout


class NetGraph(Component):
    config_defaults = {}
    configs = {}

    def __init__(self):
        # this component must be before all the normal graphs (so that
        # those other graphs are on top of the NetGraph), so its
        # order is between that of SimControl and the default (0)
        super(NetGraph, self).__init__(component_order=-5)

        # this lock ensures safety between check_for_reload() and update_code()
        self.code_lock = threading.Lock()
        self.new_code = None

        self.uids = {}
        self.parents = {}
        self.initialized_pan_and_zoom = False

    def attach(self, page, config, uid):
        super(NetGraph, self).attach(page, config, uid)
        self.layout = nengo_gui.layout.Layout(self.page.model)
        self.to_be_expanded = collections.deque([self.page.model])
        self.to_be_sent = collections.deque()

        self.networks_to_search = [self.page.model]

        try:
            self.last_modify_time = os.path.getmtime(self.page.filename)
        except OSError:
            self.last_modify_time = None
        except TypeError:  # happens if self.filename is None
            self.last_modify_time = None
        self.last_reload_check = time.time()

    def check_for_reload(self):
        if self.page.filename is not None:
            try:
                t = os.path.getmtime(self.page.filename)
                if self.last_modify_time is None or self.last_modify_time < t:
                    self.reload()
                    self.last_modify_time = t
            except OSError:
                pass

        with self.code_lock:
            new_code = self.new_code
            # the lock is in case update_code() is called between these lines
            self.new_code = None

        if new_code is not None:
            self.reload(code=new_code)

    def update_code(self, code):
        """Set new version of code to display."""
        with self.code_lock:
            self.new_code = code

    def reload(self, code=None):
        with self.page.lock:
            self._reload(code=code)

    def _reload(self, code=None):

        old_locals = self.page.last_good_locals
        old_default_labels = self.page.default_labels

        if code is None:
            with open(self.page.filename) as f:
                code = f.read()
            if self.page.code == code:
                # don't re-execute the identical code
                return
            else:
                # send the new code to the client
                self.page.ace_editor.update_code(code)

        self.page.execute(code)

        if self.page.error is not None:
            return

        name_finder = nengo_gui.NameFinder(self.page.locals, self.page.model)

        self.networks_to_search = [self.page.model]
        self.parents = {}

        removed_uids = {}
        rebuilt_objects = []

        # for each item in the old model, find the matching new item
        # for Nodes, Ensembles, and Networks, this means to find the item
        # with the same uid.  For Connections, we don't really have a uid,
        # so we use the uids of the pre and post objects.
        for uid, old_item in nengo.utils.compat.iteritems(dict(self.uids)):
            try:
                new_item = eval(uid, self.page.locals)
            except:
                new_item = None

            # check to make sure the new item's uid is the same as the
            # old item.  This is to catch situations where an old uid
            # happens to still refer to something in the new model, but that's
            # not the normal uid for that item.  For example, the uid
            # "ensembles[0]" might still refer to something even after that
            # ensemble is removed.
            new_uid = self.page.get_uid(new_item,
                        default_labels=name_finder.known_name)
            if new_uid != uid:
                new_item = None

            # find reasons to delete the object.  Any deleted object will
            # be recreated, so try to keep this to a minimum
            keep_object = True
            if new_item is None:
                keep_object = False
            elif not isinstance(new_item, old_item.__class__):
                # don't allow changing classes
                keep_object = False
            elif self.get_extra_info(new_item) != self.get_extra_info(old_item):
                keep_object = False

            if not keep_object:
                self.to_be_sent.append(dict(
                    type='remove', uid=uid))
                del self.uids[uid]
                removed_uids[old_item] = uid
                rebuilt_objects.append(uid)
            else:
                # fix aspects of the item that may have changed
                if self._reload_update_item(uid, old_item, new_item, 
                                            name_finder):
                    # something has changed about this object, so rebuild
                    # and components that use it
                    rebuilt_objects.append(uid)

                self.uids[uid] = new_item

        self.to_be_expanded.append(self.page.model)

        self.page.name_finder = name_finder
        self.page.default_labels = name_finder.known_name
        self.page.config = self.page.load_config()
        self.page.uid_prefix_counter = {}
        self.layout = nengo_gui.layout.Layout(self.page.model)
        self.page.code = code

        orphan_components = []
        rebuild_components = []

        # items that are shown in components, but not currently displayed
        #  in the NetGraph (i.e. things that are inside collapsed
        #  Networks, but whose values are being shown in a graph)
        collapsed_items = []

        removed_items = list(removed_uids.values())
        for c in self.page.components[:]:
            for item in c.code_python_args(old_default_labels):
                if item not in self.uids.keys() and item not in collapsed_items:

                    # item is a python string that is an argument to the
                    # constructor for the Component.  So it could be 'a',
                    # 'model.ensembles[3]', 'True', or even 'target=a'.
                    # We need to evaluate this string in the context of the
                    # locals dictionary and see what object it refers to
                    # so we can determine whether to rebuild this component.
                    #
                    # The following lambda should do this, handling both
                    # the normal argument case and the keyword argument case.
                    safe_eval = '(lambda *a, **b: list(a) + b.values())(%s)[0]'

                    # this Component depends on an item inside a collapsed
                    #  Network, so we need to check if that component has
                    #  changed or been removed
                    old_obj = eval(safe_eval % item, old_locals)

                    try:
                        new_obj = eval(safe_eval % item, self.page.locals)
                    except:
                        # the object this Component depends on no longer exists
                        new_obj = None

                    if new_obj is None:
                        removed_items.append(item)
                    elif not isinstance(new_obj, old_obj.__class__):
                        rebuilt_objects.append(item)
                    elif (self.get_extra_info(new_obj) != 
                          self.get_extra_info(old_obj)):
                        rebuilt_objects.append(item)

                    # add this to the list of collapsed items, so we
                    # don't recheck it if there's another Component that
                    # also depends on this
                    collapsed_items.append(item)

                if item in rebuilt_objects:
                    self.to_be_sent.append(dict(type='delete_graph',
                                                uid=c.original_id,
                                                notify_server=False))
                    rebuild_components.append(c.uid)
                    self.page.components.remove(c)
                    break
            else:
                for item in c.code_python_args(old_default_labels):
                    if item in removed_items:
                        self.to_be_sent.append(dict(type='delete_graph',
                                                    uid=c.original_id,
                                                    notify_server=False))
                        orphan_components.append(c)
                        break



        components = []
        # the old names for the old components
        component_uids = [c.uid for c in self.page.components]

        for k, v in list(self.page.locals.items()):
            if isinstance(v, nengo_gui.components.component.Component):
                # the object has been removed, so the Component should
                #  be removed as well
                if v in orphan_components:
                    continue

                # this is a Component that was previously removed,
                #  but is still in the config file, or it has to be
                #  rebuilt, so let's recover it
                if k not in component_uids:
                    self.page.add_component(v)
                    self.to_be_sent.append(dict(type='js', 
                                                code=v.javascript()))
                    components.append(v)
                    continue
                
                # otherwise, find the corresponding old Component
                index = component_uids.index(k)
                old_component = self.page.components[index]
                if isinstance(v, (nengo_gui.components.SimControlTemplate,
                                  nengo_gui.components.AceEditorTemplate,
                                  nengo_gui.components.NetGraphTemplate)):
                    # just keep these ones
                    components.append(old_component)
                else:
                    # replace these components with the newly generated ones
                    try:
                        self.page.add_component(v)
                        old_component.replace_with = v
                        v.original_id = old_component.original_id
                    except:
                        traceback.print_exc()
                        print('failed to recreate plot for %s' % v)
                    components.append(v)

        components.sort(key=lambda x: x.component_order)

        self.page.components = components

        self.page.changed = True

    def _reload_update_item(self, uid, old_item, new_item, new_name_finder):
        """Tell the client about changes to the item due to reload."""
        changed = False
        if isinstance(old_item, (nengo.Node,
                                 nengo.Ensemble,
                                 nengo.Network)):
            old_label = self.page.get_label(old_item)
            new_label = self.page.get_label(
                new_item, default_labels=new_name_finder.known_name)

            if old_label != new_label:
                self.to_be_sent.append(dict(
                    type='rename', uid=uid, name=new_label))
                changed = True
            if isinstance(old_item, nengo.Network):
                if self.page.config[old_item].expanded:
                    self.to_be_expanded.append(new_item)
                    changed = True

        elif isinstance(old_item, nengo.Connection):
            old_pre = old_item.pre_obj
            old_post = old_item.post_obj
            new_pre = new_item.pre_obj
            new_post = new_item.post_obj
            if isinstance(old_pre, nengo.ensemble.Neurons):
                old_pre = old_pre.ensemble
            if isinstance(old_post, nengo.ensemble.Neurons):
                old_post = old_post.ensemble
            if isinstance(new_pre, nengo.ensemble.Neurons):
                new_pre = new_pre.ensemble
            if isinstance(new_post, nengo.ensemble.Neurons):
                new_post = new_post.ensemble

            old_pre = self.page.get_uid(old_pre)
            old_post = self.page.get_uid(old_post)
            new_pre = self.page.get_uid(
                new_pre, default_labels=new_name_finder.known_name)
            new_post = self.page.get_uid(
                new_post, default_labels=new_name_finder.known_name)

            if new_pre != old_pre or new_post != old_post:
                # if the connection has changed, tell javascript
                pres = self.get_parents(
                    new_pre,
                    default_labels=new_name_finder.known_name)[:-1]
                posts = self.get_parents(
                    new_post,
                    default_labels=new_name_finder.known_name)[:-1]
                self.to_be_sent.append(dict(
                    type='reconnect', uid=uid,
                    pres=pres, posts=posts))
                changed = True
        return changed

    def get_parents(self, uid, default_labels=None):
        while uid not in self.parents:
            net = self.networks_to_search.pop(0)
            net_uid = self.page.get_uid(net, default_labels=default_labels)
            for n in net.nodes:
                n_uid = self.page.get_uid(n, default_labels=default_labels)
                self.parents[n_uid] = net_uid
            for e in net.ensembles:
                e_uid = self.page.get_uid(e, default_labels=default_labels)
                self.parents[e_uid] = net_uid
            for n in net.networks:
                n_uid = self.page.get_uid(n, default_labels=default_labels)
                self.parents[n_uid] = net_uid
                self.networks_to_search.append(n)
        parents = [uid]
        while parents[-1] in self.parents:
            parents.append(self.parents[parents[-1]])
        return parents

    def modified_config(self):
        self.page.modified_config()

    def update_client(self, client):
        now = time.time()
        if now > self.last_reload_check + 0.5:
            self.check_for_reload()
            self.last_reload_check = now

        if not self.initialized_pan_and_zoom:
            self.send_pan_and_zoom(client)
            self.initialized_pan_and_zoom = True

        while len(self.to_be_sent) > 0:
            info = self.to_be_sent.popleft()
            client.write(json.dumps(info))

        if len(self.to_be_expanded) > 0:
            with self.page.lock:
                network = self.to_be_expanded.popleft()
                self.expand_network(network, client)

    def javascript(self):
        return 'new Nengo.NetGraph(main, {uid:"%s"});' % id(self)

    def message(self, msg):
        try:
            info = json.loads(msg)
        except ValueError:
            print('invalid message', repr(msg))
            return
        action = info.get('act', None)
        undo = info.get('undo', None)
        if action is not None:
            del info['act']
            if action in ('auto_expand', 'auto_collapse'):
                getattr(self, 'act_' + action[5:])(**info)
            elif action in ('pan', 'zoom', 'create_modal'):
                # These should not use the undo stack
                getattr(self, 'act_' + action)(**info)
            else:
                act = nengo_gui.user_action.create_action(action, self, **info)
                self.page.undo_stack.append([act])
                del self.page.redo_stack[:]
        elif undo is not None:
            if undo == '1':
                self.undo()
            else:
                self.redo()
        else:
            print('received message', msg)

    def undo(self):
        if self.page.undo_stack:
            action = self.page.undo_stack.pop()
            re = []
            for act in action:
                act.undo()
                re.insert(0, act)
            self.page.redo_stack.append(re)

    def redo(self):
        if self.page.redo_stack:
            action = self.page.redo_stack.pop()
            un = []
            for act in action:
                act.apply()
                un.insert(0, act)
            self.page.undo_stack.append(un)

    def act_expand(self, uid):
        net = self.uids[uid]
        self.to_be_expanded.append(net)
        self.page.config[net].expanded = True
        self.modified_config()

    def act_collapse(self, uid):
        net = self.uids[uid]
        self.page.config[net].expanded = False
        self.remove_uids(net)
        self.modified_config()

    def remove_uids(self, net):
        for items in [net.ensembles, net.networks, net.nodes, net.connections]:
            for item in items:
                uid = self.page.get_uid(item)
                if uid in self.uids:
                    del self.uids[uid]
        for n in net.networks:
            self.remove_uids(n)

    def act_pan(self, x, y):
        self.page.config[self.page.model].pos = x, y
        self.modified_config()

    def act_zoom(self, scale, x, y):
        self.page.config[self.page.model].size = scale, scale
        self.page.config[self.page.model].pos = x, y
        self.modified_config()

    def act_create_modal(self, uid, **info):
        js = infomodal(self, uid, **info)
        self.to_be_sent.append(dict(type='js', code=js))

    def expand_network(self, network, client):
        if not self.page.config[network].has_layout:
            pos = self.layout.make_layout(network)
            for obj, layout in pos.items():
                self.page.config[obj].pos = layout['y'], layout['x']
                self.page.config[obj].size = layout['h'] / 2, layout['w'] / 2
            self.page.config[network].has_layout = True

        if network is self.page.model:
            parent = None
        else:
            parent = self.page.get_uid(network)
        for ens in network.ensembles:
            self.create_object(client, ens, type='ens', parent=parent)
        for node in network.nodes:
            self.create_object(client, node, type='node', parent=parent)
        for net in network.networks:
            self.create_object(client, net, type='net', parent=parent)
        for conn in network.connections:
            self.create_connection(client, conn, parent=parent)
        self.page.config[network].expanded = True

    def create_object(self, client, obj, type, parent):
        uid = self.page.get_uid(obj)
        if uid in self.uids:
            return

        pos = self.page.config[obj].pos
        if pos is None:
            import random
            pos = random.uniform(0, 1), random.uniform(0, 1)
            self.page.config[obj].pos = pos
        size = self.page.config[obj].size
        if size is None:
            size = (0.1, 0.1)
            self.page.config[obj].size = size
        label = self.page.get_label(obj)
        self.uids[uid] = obj
        info = dict(uid=uid, label=label, pos=pos, type=type, size=size,
                    parent=parent)
        if type == 'net':
            info['expanded'] = self.page.config[obj].expanded
        info.update(self.get_extra_info(obj))

        client.write(json.dumps(info))

    def get_extra_info(self, obj):
        '''Determine helper information for each nengo object.

        This is used by the client side to configure the display.  It is also
        used by the reload() code to determine if a NetGraph object should
        be recreated.
        '''
        info = {}
        if isinstance(obj, nengo.Node):
            if obj.output is None:
                info['passthrough'] = True
            if callable(obj.output) and hasattr(obj.output, '_nengo_html_'):
                info['html'] = True
            info['dimensions'] = int(obj.size_out)
        elif isinstance(obj, nengo.Ensemble):
            info['dimensions'] = int(obj.size_out)
            info['n_neurons'] = int(obj.n_neurons)
        elif Value.default_output(obj) is not None:
            info['default_output'] = True

        info['sp_targets'] = (
            nengo_gui.components.spa_plot.SpaPlot.applicable_targets(obj))
        return info

    def send_pan_and_zoom(self, client):
        pan = self.page.config[self.page.model].pos
        if pan is None:
            pan = 0, 0
        zoom = self.page.config[self.page.model].size
        if zoom is None:
            zoom = 1.0
        else:
            zoom = zoom[0]
        client.write(json.dumps(dict(type='pan', pan=pan)))
        client.write(json.dumps(dict(type='zoom', zoom=zoom)))

    def create_connection(self, client, conn, parent):
        uid = self.page.get_uid(conn)
        if uid in self.uids:
            return
        pre = conn.pre_obj
        if isinstance(pre, nengo.ensemble.Neurons):
            pre = pre.ensemble
        post = conn.post_obj
        if isinstance(post, nengo.ensemble.Neurons):
            post = post.ensemble
        pre = self.page.get_uid(pre)
        post = self.page.get_uid(post)
        self.uids[uid] = conn
        pres = self.get_parents(pre)[:-1]
        posts = self.get_parents(post)[:-1]
        info = dict(uid=uid, pre=pres, post=posts, type='conn', parent=parent)
        client.write(json.dumps(info))



# Tutorial 2: Two neurons

# Now we show what happens with two neurons.  Notice that the neurons
# respond very differently to the two inputs.  One neuron responds more
# strongly to positive values, while the other responds more to negative
# values.  This is typical of real neurons (sometimes called "on" and "off"
# neurons).

# Given these two neurons, more information is available about what the input
# is.  The bottom graph shows the "decoded" value found by taking the output
# of both neurons and combining them.  Notice that these two neurons do a
# slightly better job of representing the input than just one neuron.


import nengo

model = nengo.Network()
with model:
    stim = nengo.Node(0)
    ens = nengo.Ensemble(n_neurons=2, dimensions=1, seed=5)
    nengo.Connection(stim, ens)

    # include these lines if the simulation is too fast
    import time
    slowdown = nengo.Node(lambda t: time.sleep(0.001))



from .component import Component
from .slider import Slider
from .value import Value
from .xyvalue import XYValue
from .sim_control import SimControl
from .raster import Raster
from .voltage import Voltage
from .pointer import Pointer
from .netgraph import NetGraph
from .ace_editor import AceEditor
from .spa_similarity import SpaSimilarity
from .htmlview import HTMLView
from .spike_grid import SpikeGrid

# Old versions of the .cfg files used Templates which had slightly different
# names than the Components currently use.  This code allows us to
# successfully parse those old .cfg files
SliderTemplate = Slider
ValueTemplate = Value
XYValueTemplate = XYValue
SimControlTemplate = SimControl
RasterTemplate = Raster
VoltageTemplate = Voltage
PointerTemplate = Pointer
NetGraphTemplate = NetGraph
AceEditorTemplate = AceEditor



import struct
import collections

import nengo
import numpy as np

from nengo_gui.components.component import Component


class HTMLView(Component):
    def __init__(self, obj):
        super(HTMLView, self).__init__()
        self.obj = obj
        self.obj_output = obj.output
        self.data = collections.deque()

    def attach(self, page, config, uid):
        super(HTMLView, self).attach(page, config, uid)
        self.label = page.get_label(self.obj)

    def add_nengo_objects(self, page):
        with page.model:
            self.obj.output = self.gather_data

    def remove_nengo_objects(self, page):
        self.obj.output = self.obj_output

    def gather_data(self, t, *x):
        value = self.obj_output(t, *x)
        data = '%g %s' % (t, self.obj_output._nengo_html_)
        self.data.append(data)
        return value

    def update_client(self, client):
        while len(self.data) > 0:
            item = self.data.popleft()
            client.write(item)

    def javascript(self):
        info = dict(uid=id(self), label=self.label)
        json = self.javascript_config(info)
        return 'new Nengo.HTMLView(main, sim, %s);' % json

    def code_python_args(self, uids):
        return [uids[self.obj]]



import os
import warnings

import nengo_gui.page
import nengo_gui.server


class GUI(object):
    """The master server object for running nengo_gui.

    Parameters
    ----------

    filename : str, optional
        Name of the default file to open when the gui starts.  If this is
        None and the model is also None, the default is 'examples/default.py'
        in the installation directory.
    model : nengo.Network, optional
        The nengo.Network to show.  If this is None, the model will be taken
        from locals['model'] (if locals is not None) or from executing the
        file given by filename
    locals : dict, optional
        The locals() dictionary after defining the model.  If this is not
        None, it will be used to help find readable labels for all the
        components of the model.  Otherwise, the resulting locals()
        dictionary after executing the script given by filename will be used.
    cfg : str, optional
        The filename to use for the config file.  If this is None, the
        default of filename + '.cfg' will be used.
    interactive : bool, optional
        If this is False, the nengo_gui is running inside some other system,
        such as IPython notebook.  The server will not automatically shut
        down, error messages will not be printed, and the code editor will
        not be shown.
    allow_file_change : bool, optional
        If this is False, the "open file" button will be disabled.
    backend : str, optional
        The default backend to use.
    """

    def __init__(self, filename=None, model=None, locals=None,
                 cfg=None, interactive=True, allow_file_change=True,
                 backend='nengo'):

        # no starting a GUI inside a script inside a GUI
        if nengo_gui.exec_env.is_executing():
            raise nengo_gui.exec_env.StartedGUIException()

        # the list of running Pages
        self.pages = []

        # a mapping from uids to Components for all running Pages.
        # this is used to connect the websockets to the appropriate Component
        self.component_uids = {}

        # should the GUI shut down
        self.finished = False

        if filename is None and model is None:
            filename = os.path.join(nengo_gui.__path__[0],
                                    'examples',
                                    'default.py')
        if filename is not None:
            try:
                filename = os.path.relpath(filename)
            except ValueError:
                # happens on Windows if filename is on a different
                # drive than the current directory
                filename = filename

        if model is None and locals is not None:
            model = locals.get('model', None)

        self.filename = filename
        self.model = model
        self.locals = locals
        self.allow_file_change = allow_file_change
        self.filename_cfg = cfg
        self.interactive = interactive
        self.backend = backend

    def start(self, port=8080, browser=True, password=None):
        """Start the web server"""
        print("Starting nengo_gui server at http://localhost:%d" % port)
        if password is not None:
            nengo_gui.server.Server.add_user('', password)
            addr = ''
        else:
            addr = 'localhost'
        nengo_gui.server.Server.start(self, port=port, browser=browser,
                                      addr=addr, interactive=self.interactive)

    def prepare_server(self, port=8080, browser=True):
        return nengo_gui.server.Server.prepare_server(
            self, port=port, browser=browser)

    def begin_lifecycle(self, server):
        nengo_gui.server.Server.begin_lifecycle(
            server, interactive=self.interactive)

    def create_page(self, filename, reset_cfg=False):
        """Create a new Page with this configuration"""
        page = nengo_gui.page.Page(self, filename=filename, reset_cfg=reset_cfg)
        self.pages.append(page)
        return page

    def remove_page(self, page):
        self.pages.remove(page)

    def count_pages(self):
        return len(self.pages)



import time
import timeit
import struct
import traceback

import numpy as np
import nengo
import os
import os.path
import json

from nengo_gui.components.component import Component
import nengo_gui.exec_env

class SimControl(Component):
    config_defaults = dict(shown_time=0.5, kept_time=4.0)
    def __init__(self, dt=0.001):
        # this component must be the very first one defined, so
        # its component_order is the smallest overall
        super(SimControl, self).__init__(component_order=-10)
        self.paused = True
        self.last_tick = None
        self.rate = 0.0
        self.model_dt = dt
        self.rate_tau = 1.0
        self.last_send_rate = None
        self.sim_ticks = 0
        self.skipped = 1
        self.time = 0.0
        self.last_status = None
        self.next_ping_time = None
        self.send_config_options = False
        self.reset_inform = False

    def attach(self, page, config, uid):
        super(SimControl, self).attach(page, config, uid)
        self.shown_time = config.shown_time
        self.kept_time = config.kept_time


    def add_nengo_objects(self, page):
        with page.model:
            self.node = nengo.Node(self.control, size_out=0)

    def remove_nengo_objects(self, page):
        page.model.nodes.remove(self.node)

    def finish(self):
        self.page.finish()

    def control(self, t):
        self.actual_model_dt = t - self.time
        self.time = t
        self.sim_ticks += 1

        now = timeit.default_timer()
        if self.last_tick is not None:
            dt = now - self.last_tick
            if dt == 0:
                self.skipped += 1
            else:
                rate = self.actual_model_dt * self.skipped / dt
                decay = np.exp(-dt / self.rate_tau)
                self.rate *= decay
                self.rate += (1 - decay) * rate
                self.skipped = 1

        self.last_tick = now

        while self.paused and self.page.sim is not None:
            time.sleep(0.01)
            self.last_tick = None

    def config_settings(self, data):
        for i in data:
            print(i)

    def update_client(self, client):
        now = time.time()
        # send off a ping now and then so we'll notice when connection closes
        if self.next_ping_time is None or now > self.next_ping_time:
            client.write('', ping=True)
            self.next_ping_time = now + 2.0

        if self.page.changed:
            self.paused = True
            self.page.sim = None
            self.page.changed = False
        if not self.paused or self.reset_inform:
            client.write(struct.pack('<ff', self.time, self.rate), binary=True)
            self.reset_inform = False
        status = self.get_status()
        if status != self.last_status:
            client.write('status:%s' % status)
            self.last_status = status
        if self.send_config_options == True:
            client.write('sims:' + self.backend_options_html())
            client.write('config' +
                'Nengo.Toolbar.prototype.config_modal_show();')
            self.send_config_options = False

    def get_status(self):
        if self.paused:
            return 'paused'
        elif self.page.sim is None:
            if self.page.error is None:
                return 'building'
            else:
                return 'build_error'
        else:
            return 'running'

    def javascript(self):
        info = dict(uid=id(self))
        fn = json.dumps(self.page.filename)
        js = self.javascript_config(info)
        return ('sim = new Nengo.SimControl(control, %s);\n'
                'toolbar = new Nengo.Toolbar(%s); ' % (js, fn))

    def message(self, msg):
        if msg == 'pause':
            self.paused = True
        elif msg == 'config':
            self.send_config_options = True
        elif msg == 'continue':
            if self.page.sim is None:
                self.page.rebuild = True
            self.paused = False
        elif msg == 'reset':
            self.paused = True
            self.time = 0
            self.rate = 0
            self.reset_inform = True
            self.page.sim = None
        elif msg[:8] == 'backend:':
            self.page.backend = msg[8:]
            self.page.changed = True

    def backend_options_html(self):
        items = []
        for module in nengo_gui.exec_env.discover_backends():
            if module == self.page.backend:
                selected = ' selected'
            else:
                selected = ''
            item = '<option %s>%s</option>' % (selected, module)
            items.append(item)
        return ''.join(items)



"""One-off functions that create Javascript calls"""

import json

import numpy as np

import nengo

from .static_plots import tuning_curve_plot, response_curve_plot
from .static_plots import node_output_plot

def infomodal(ng, uid, **args):
    obj = ng.uids[uid]
    if isinstance(obj, nengo.Ensemble):
        return ensemble_infomodal(ng, uid=uid, **args)
    elif isinstance(obj, nengo.Node):
        return node_infomodal(ng, uid=uid, **args)
    elif isinstance(obj, nengo.Network):
        return net_infomodal(ng, uid=uid, **args)
    else:
        raise NotImplementedError()

def add_modal_title_js(title_text):
    return 'Nengo.modal.title("%s");' % (title_text)

def add_modal_footer_js(footer_text):
    return 'Nengo.modal.footer("%s");' % (footer_text)

def show_modal_js():
    return 'Nengo.modal.show();'

def ensemble_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    ens = ng.uids[uid]

    params = [(attr, str(getattr(ens, attr))) for attr in (
        'n_neurons', 'dimensions', 'radius', 'encoders', 'intercepts',
        'max_rates', 'eval_points', 'n_eval_points', 'neuron_type',
        'noise', 'seed') if getattr(ens, attr) is not None]

    if ng.page.sim is None:
        plots = ("Simulation not yet running. "
                 "Start a simulation to see plots.")
    else:
        plots = []
        plots.append(response_curve_plot(ens, ng.page.sim))
        plots.append(tuning_curve_plot(ens, ng.page.sim))

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = ['Nengo.modal.title("Details for \'%s\'");' % ng.page.get_label(ens)]
    js.append('Nengo.modal.footer("close");')
    js.append('Nengo.modal.ensemble_body("%s", %s, %s, %s);' % (
        uid, json.dumps(params), json.dumps(plots), json.dumps(conninfo)))
    js.append('Nengo.modal.show();')
    return '\n'.join(js)

def node_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    node = ng.uids[uid]

    params = [(attr, str(getattr(node, attr))) for attr in (
        'output', 'size_in', 'size_out') if getattr(node, attr) is not None]

    plots = [node_output_plot(node)]

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = [add_modal_title_js("Details for \'%s\'" % (
        ng.page.get_label(node)))]
    js.append(add_modal_footer_js('close'))
    js.append('Nengo.modal.node_body("%s", %s, %s, %s);' % (
        uid, json.dumps(params), json.dumps(plots), json.dumps(conninfo)))
    js.append(show_modal_js())
    return '\n'.join(js)


def conn_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    conninfo = {}

    conninfo["obj_type"] = {}
    conninfo["func"] = {}
    conninfo["fan"] = {}

    def get_conn_func_str(conn_uid):
        return (str(ng.uids[conn_uid].function)
                if ng.uids[conn_uid].function is not None
                else "Identity function")

    def get_obj_info(nengo_obj):
        if isinstance(nengo_obj, nengo.Node):
            if nengo_obj.output is None:
                return "passthrough", "> 0"
            else:
                return "node", "0"
        elif isinstance(nengo_obj, nengo.Ensemble):
            return "ens", str(nengo_obj.n_neurons)
        elif isinstance(nengo_obj, nengo.Network):
            return "net", str(sum(map(lambda e: e.n_neurons,
                                      nengo_obj.all_ensembles)))
        else:
            return "err", "0"

    for conn_uid in conn_in_uids:
        conninfo["obj_type"][conn_uid], conninfo["fan"][conn_uid] = \
            get_obj_info(ng.uids[conn_uid].pre_obj)
        conninfo["func"][conn_uid] = get_conn_func_str(conn_uid)

    for conn_uid in conn_out_uids:
        conninfo["obj_type"][conn_uid], conninfo["fan"][conn_uid] = \
            get_obj_info(ng.uids[conn_uid].post_obj)
        conninfo["func"][conn_uid] = get_conn_func_str(conn_uid)

    return conninfo


def net_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    net = ng.uids[uid]

    stats = []
    stats.append({
        'title': "In this network",
        'stats': [
            ('Number of ensembles', len(net.ensembles)),
            ('Total number of neurons',
             sum(e.n_neurons for e in net.ensembles)),
            ('Number of nodes', len(net.nodes)),
            ('Number of connections', len(net.connections)),
            ('Number of subnetworks', len(net.networks)),
        ]})
    stats.append({
        'title': "In this network and subnetworks",
        'stats': [
            ('Number of ensembles', len(net.all_ensembles)),
            ('Total number of neurons',
             sum(e.n_neurons for e in net.all_ensembles)),
            ('Number of nodes', len(net.all_nodes)),
            ('Number of connections', len(net.all_connections)),
            ('Number of subnetworks', len(net.all_networks)),
        ]})

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = [add_modal_title_js("Details for \'%s\'") % (
        ng.page.get_label(net))]
    js.append(add_modal_footer_js('close'))
    js.append('Nengo.modal.net_body("%s", %s, %s);' % (
        uid, json.dumps(stats), json.dumps(conninfo)))
    js.append(show_modal_js())
    return '\n'.join(js)
